import unittest

import numpy as np

import pystan


class TestPosteriorMeans(unittest.TestCase):
    """Test get_posterior_mean"""

    @classmethod
    def setUpClass(cls):
        model_code = """
        data {
            int<lower=0> n_subjects; // number of subjects
            int<lower=0,upper=40> score[n_subjects]; // UPSIT scores
            int<lower=0,upper=40> n_questions; // number of questions
        }
        parameters {
            real rating[n_subjects];
        }
        transformed parameters {
            real p[n_subjects];
            for (j in 1:n_subjects) {
                p[j] <- exp(rating[j])/(1+exp(rating[j])); // Logistic function
                }
        }
        model {
            rating ~ normal(0, 1);
            score ~ binomial(40, p);
        }
        """

        cls.sm = pystan.StanModel(model_code=model_code)
        cls.data = {'n_questions': 40,
                    'n_subjects': 92,
                    'score': np.random.randint(0, 40, 92)}

    def test_posterior_means(self):
        sm = self.sm
        data = self.data
        fit = sm.sampling(data, iter=100)
        self.assertIsNotNone(fit.get_posterior_mean())



import unittest

import numpy as np

import pystan


class TestUserInits(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        model_code = """
            data {
              real x;
            }
            parameters {
              real mu;
            }
            model {
              x ~ normal(mu,1);
            }
        """
        cls.data = dict(x=2)
        cls.model = pystan.StanModel(model_code=model_code)

    def test_user_init(self):
        model, data = self.model, self.data
        fit1 = model.sampling(iter=10, chains=1, seed=2, data=data, init=[dict(mu=4)], warmup=0)
        self.assertEqual(fit1.get_inits()[0]['mu'], 4)
        fit2 = model.sampling(iter=10, chains=1, seed=2, data=data, init=[dict(mu=400)], warmup=0)
        self.assertEqual(fit2.get_inits()[0]['mu'], 400)
        self.assertFalse(all(fit1.extract()['mu'] == fit2.extract()['mu']))

    def test_user_initfun(self):
        model, data = self.model, self.data

        def make_inits(chain_id):
            return dict(mu=chain_id)

        fit1 = model.sampling(iter=10, chains=4, seed=2, data=data, init=make_inits, warmup=0)
        for i, inits in enumerate(fit1.get_inits()):
            self.assertEqual(inits['mu'], i)

    def test_user_initfun_chainid(self):
        model, data = self.model, self.data

        def make_inits(chain_id):
            return dict(mu=chain_id)

        chain_id = [9, 10, 11, 12]
        fit1 = model.sampling(iter=10, chains=4, seed=2, data=data,
                              init=make_inits, warmup=0, chain_id=chain_id)
        for i, inits in zip(chain_id, fit1.get_inits()):
            self.assertEqual(inits['mu'], i)

    def test_user_init_unspecified(self):
        model_code = """
        data {
          real x;
        }
        parameters {
          real mu;
          real<lower=0> sigma;
        }
        model {
          x ~ normal(mu, sigma);
        }
        """
        data = self.data
        # NOTE: we are only specifying 'mu' and not 'sigma' (partial inits)
        fit = pystan.stan(model_code=model_code, iter=10, chains=1, seed=2, data=data, init=[dict(mu=4)], warmup=0)
        self.assertIsNotNone(fit)


class TestUserInitsMatrix(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        model_code = """
            data {
              int<lower=2> K;
              int<lower=1> D;
            }
            parameters {
              matrix[K,D] beta;
            }
            model {
            for (k in 1:K)
                for (d in 1:D)
                  beta[k,d] ~ normal(if_else(d==2,100, 0),1);
           }
        """
        cls.model = pystan.StanModel(model_code=model_code)
        cls.data = dict(K=3, D=4)

    def test_user_init(self):
        model, data = self.model, self.data
        beta = np.ones((data['K'], data['D']))
        fit1 = model.sampling(iter=10, chains=1, seed=2,
                              data=data, init=[dict(beta=beta)], warmup=0)
        np.testing.assert_equal(fit1.get_inits()[0]['beta'], beta)
        beta = 5 * np.ones((data['K'], data['D']))
        fit2 = model.sampling(iter=10, chains=1, seed=2,
                              data=data, init=[dict(beta=beta)], warmup=0)
        np.testing.assert_equal(fit2.get_inits()[0]['beta'], beta)

    def test_user_initfun(self):
        model, data = self.model, self.data

        beta = np.ones((data['K'], data['D']))

        def make_inits(chain_id):
            return dict(beta=beta * chain_id)

        fit1 = model.sampling(iter=10, chains=4, seed=2, data=data, init=make_inits, warmup=0)
        for i, inits in enumerate(fit1.get_inits()):
            np.testing.assert_equal(beta * i, inits['beta'])



import unittest
import numpy as np
import pystan

class TestLookup(unittest.TestCase):
    def test_lookup(self):
        cbind = pystan.lookup("R.cbind")["StanFunction"]
        hstack = pystan.lookup("numpy.hstack")["StanFunction"]
        cbind_err = pystan.lookup("R.cbindd")["StanFunction"]
        cbind_2 = pystan.lookup("R.cbind", 1.0)["StanFunction"]
        hstack_2 = pystan.lookup("numpy.hstack", 1.0)["StanFunction"]
        for i in range(len(cbind)):
            self.assertEqual(cbind[i], "append_col")
            self.assertEqual(cbind[i], hstack[i])
            self.assertEqual(cbind[i], cbind_err[i])
            self.assertEqual(cbind[i], cbind_2[i])
            self.assertEqual(cbind[i], hstack_2[i])
        normal = pystan.lookup("scipy.stats.norm")["StanFunction"]
        for i in range(len(normal)):
            self.assertEqual("normal", normal[i][:6])
        poisson = pystan.lookup("scipy.stats.poisson")["StanFunction"]
        for i in range(len(poisson)):
            self.assertEqual("poisson", poisson[i][:7])
        wrongf = pystan.lookup("someverycrazyfunctionyouwontfind", 1.0)
        self.assertTrue(wrongf is None)



#!/usr/bin/python

"""
Replacement for runtest target in Makefile.

Call script with '-h' as an option to see a helpful message.
"""

from __future__ import print_function
from argparse import ArgumentParser, RawTextHelpFormatter
import os
import os.path
import platform
import subprocess
import sys
import time

winsfx = ".exe"
testsfx = "_test.cpp"
batchSize = 25


def processCLIArgs():
    """
    Define and process the command line interface to the runTests.py script.
    """
    cli_description = "Generate and run stan math library tests."
    cli_epilog = "See more information at: https://github.com/stan-dev/math"

    parser = ArgumentParser(description=cli_description,
                            epilog=cli_epilog,
                            formatter_class=RawTextHelpFormatter)

    # Now define all the rules of the command line args and opts
    parser.add_argument("-j", metavar="N", type=int, default=1,
                        help="number of cores for make to use")

    tests_help_msg = "The path(s) to the test case(s) to run.\n"
    tests_help_msg += "Example: 'test/unit', 'test/prob', and/or\n"
    tests_help_msg += "         'test/unit/math/prim/scal/fun/abs_test.cpp'"
    parser.add_argument("tests", nargs="+", type=str,
                        help=tests_help_msg)

    parser.add_argument("-d", "--debug", dest="debug", action="store_true",
                        help="request additional script debugging output.")

    # And parse the command line against those rules
    return parser.parse_args()


def stopErr(msg, returncode):
    """Report an error message to stderr and exit with a given code."""
    sys.stderr.write('%s\n' % msg)
    sys.stderr.write('exit now (%s)\n' % time.strftime('%x %X %Z'))
    sys.exit(returncode)


def isWin():
    if (platform.system().lower().startswith("windows")
            or os.name.lower().startswith("windows")):
        return True
    return False


def mungeName(name):
    """Set up the makefile target name"""
    if (name.endswith(testsfx)):
        name = name.replace(testsfx, "_test")

        if (isWin()):
            name += winsfx
            name = name.replace("\\", "/")

    return name


def doCommand(command):
    """Run command as a shell command and report/exit on errors."""
    print("------------------------------------------------------------")
    print("%s" % command)
    p1 = subprocess.Popen(command, shell=True)
    p1.wait()
    if (not(p1.returncode is None) and not(p1.returncode == 0)):
        stopErr('%s failed' % command, p1.returncode)


def generateTests(j):
    """Generate all tests and pass along the j parameter to make."""
    if j is None:
        command = 'make generate-tests -s'
    else:
        command = 'make -j%d generate-tests -s' % j
    doCommand(command)


def makeTest(name, j):
    """Run the make command for a given single test."""
    target = mungeName(name)
    if j is None:
        command = 'make %s' % target
    else:
        command = 'make -j%d %s' % (j, target)
    doCommand(command)


def makeTests(dirname, filenames, j, debug):
    targets = list()
    for name in filenames:
        if (not name.endswith(testsfx)):
            continue

        target = "/".join([dirname, name])
        target = mungeName(target)
        targets.append(target)

    if (len(targets) > 0):
        if (debug):
            print('# targets: %d' % len(targets))
        startIdx = 0
        endIdx = batchSize

        while (startIdx < len(targets)):
            if j is None:
                command = 'make %s' % ' '.join(targets[startIdx:endIdx])
            else:
                command = 'make -j%d %s' % (j, ' '.join(targets[startIdx:endIdx]))

            if (debug):
                print('start %d, end %d' % (startIdx, endIdx))
                print(command)

            doCommand(command)
            startIdx = endIdx
            endIdx = startIdx + batchSize

            if (endIdx > len(targets)):
                endIdx = len(targets)


def runTest(name):
    executable = mungeName(name).replace("/", os.sep)
    xml = mungeName(name).replace(winsfx, "")
    command = '%s --gtest_output="xml:%s.xml"' % (executable, xml)
    doCommand(command)


def main():
    inputs = processCLIArgs()

    # pass 0: generate all auto-generated tests
    if any(['test/prob' in arg for arg in inputs.tests]):
        generateTests(inputs.j)

    # pass 1: call make to compile test targets
    for testname in inputs.tests:
        # Ensure that the test actually exists at all
        if (not(os.path.exists(testname))):
            stopErr('%s: no such file or directory' % testname, -1)

        # If it exists and is a not a directory, run that one test
        if (not(os.path.isdir(testname))):
            if (not(testname.endswith(testsfx))):
                stopErr('%s: not a testfile' % testname, -1)

            if (inputs.debug):
                print("make single test: %s" % testname)

            makeTest(testname, inputs.j)

        # If it exists and is a directory, run all tests inside
        else:
            for root, dirs, files in os.walk(testname):
                if (inputs.debug):
                    print("make root: %s" % root)

                makeTests(root, files, inputs.j, inputs.debug)

    # pass 2: run test targets
    for testname in inputs.tests:
        if (not(os.path.isdir(testname))):
            if (inputs.debug):
                print("run single test: %s" % testname)
            runTest(testname)
        else:
            for root, dirs, files in os.walk(testname):
                for name in files:
                    if (name.endswith(testsfx)):
                        if (inputs.debug):
                            print("run dir,test: %s,%s" % (root, name))
                        runTest(os.sep.join([root, name]))


if __name__ == "__main__":
    main()



import unittest

import pystan
import pystan.chains
import pystan._chains

# NOTE: This test is fragile because the default sampling algorithm used by Stan
# may change in significant ways.


class TestESS(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        model_code = 'parameters {real y;} model {y ~ normal(0,1);}'
        model = pystan.StanModel(model_code=model_code)
        cls.fits = [model.sampling(iter=4000, chains=2, seed=i) for i in range(10)]

    def test_ess(self):
        sims = [fit.sim for fit in self.fits]
        esses = [pystan.chains.ess(sim, sim['fnames_oi'].index('y')) for sim in sims]
        # in Stan 2.15 the default acceptance rate was changed, ess dropped
        self.assertGreater(sum(esses) / len(esses), 1200)
